dofile_once("data/scripts/lib/utilities.lua")
dofile_once( "data/scripts/perks/perk_list.lua" )

--Returns the component ID of the perk counter.
function GetPerkCounter(aEntity)
	--GamePrint("Getting Perk Counter Component: " .. aPerkID)
	local components = EntityGetComponent(aEntity, "VariableStorageComponent")

	if components ~= nil then
		--GamePrint("Components != nil")
		--GamePrint("Looking for counter")
		for i,component_id in ipairs(components) do
			local comp_name = ComponentGetValue(component_id, "name")
			--GamePrint("Found: " .. comp_name)
			if comp_name == "counter" then
				--GamePrint("Returning: " .. tostring(component_id))
				return component_id
			end
		end
	else
		return nil
	end
end

--Returns the component ID of the perk counter.
function GetPerkUI(aEntity)
	--GamePrint("Getting Perk UI Icon Component")
	local component = EntityGetFirstComponent(aEntity, "UIIconComponent")

	if component ~= nil then
		--GamePrint("Component != nil")
		--GamePrint("Returning: " .. component)
		return component
	else
		--GamePrint("Component = nil")
		return nil
	end
end

function GetPerkEntity(aPerkID, aParentEntity)
	--GamePrint("Getting Perk Entity: " .. aPerkID)
	local entity_name = aPerkID .. "_entity"
	local entity_children = EntityGetAllChildren(aParentEntity)
	
	for i,entity_id in ipairs(entity_children) do
		--GamePrint("Found: " .. EntityGetName(entity_id))
		if EntityGetName(entity_id) == entity_name then
			--GamePrint("Got Entity " .. entity_id)
			return entity_id
		end
	end
	
	return nil
end

perk_pickup = function(entity_item, entity_who_picked, item_name, do_cosmetic_fx, kill_other_perks)
    local pos_x, pos_y = EntityGetTransform( entity_item )
	
	local perk_name = "PERK_NAME_NOT_DEFINED"
	local perk_desc = "PERK_DESCRIPTION_NOT_DEFINED"
	
	edit_component( entity_item, "ItemComponent", function(comp,vars)
		perk_name = ComponentGetValue( comp, "item_name")
		perk_desc = ComponentGetValue( comp, "ui_description")
	end)

	local perk_id = ""
	edit_component( entity_item, "VariableStorageComponent", function(comp,vars)
		perk_id = ComponentGetValue( comp, "value_string" )
	end)

	local perk_data = get_perk_with_id( perk_list, perk_id )
	if perk_data == nil then
		return
	end

	-- load perk for entity_who_picked -----------------------------------

	local flag_name = get_perk_picked_flag_name( perk_id ) 
	GameAddFlagRun( flag_name )
	AddFlagPersistent( string.lower( flag_name ) )

	-- add game effect
	if perk_data.game_effect ~= nil then
		local game_effect_comp = GetGameEffectLoadTo( entity_who_picked, perk_data.game_effect, true )
		if game_effect_comp ~= nil then
			ComponentSetValue( game_effect_comp, "frames", "-1" )
		end
	end

	if perk_data.func ~= nil then
		perk_data.func( entity_item, entity_who_picked, item_name )
	end
	
	--Get the perk entity if it exists.
	local perk_entity = GetPerkEntity(perk_data.id, entity_who_picked)
	
	--Check if the entity exists.
	if perk_entity ~= nil then
		--Get the counter
		local perk_counter = GetPerkCounter(perk_entity)
		--Get the count.
		local perk_count = ComponentGetValue(perk_counter, "value_int")
		--GamePrint("Current count: " .. perk_count)
		local new_count = perk_count
		
		if perk_counter ~= nil then
			--Up the counter for the perk.
			new_count = perk_count + 1
			ComponentSetValue(perk_counter, "value_int", new_count)
			--GamePrint("New perk count: " .. tostring(new_count))
		--else
			--GamePrint("perk_counter = nil")
		end
		
		--Get the UI Icon
		local perk_ui = GetPerkUI(perk_entity)
		
		if perk_ui ~= nil then
			--Edit the name of the UI element to reflect the counter.
			--TODO: GUI Counter.
			local new_name = GameTextGetTranslatedOrNot(perk_data.ui_name) .. " (x" .. tostring(new_count) .. ")"
			ComponentSetValue(perk_ui, "name", new_name)
			--GamePrint("New UI Name: " .. perk_data.ui_name .. " (x" .. tostring(new_count) .. ")")
		--else
			--GamePrint("perk_ui = nil")
		end
	else
		--Create a perk entity to track the count as well as hold the UI component.
		local perk_entity = EntityCreateNew( "" )
		--GamePrint("Adding Counter to: " .. tostring(entity_who_picked))
		--GamePrint("Adding new Perk Counter component")
		EntityAddComponent(perk_entity, "VariableStorageComponent", 
		{ 
			name = "counter",
			value_int = 1
		})
		--GamePrint("Adding UIIconComponent")
		EntityAddComponent(perk_entity, "UIIconComponent", 
		{ 
			name = perk_data.ui_name,
			description = perk_data.ui_description,
			icon_sprite_file = perk_data.ui_icon
		})
		--Set the name of the Entity to reflect the perk being tracked.
		EntitySetName(perk_entity, perk_data.id .. "_entity")
		--Add the UI element to the entity_who_picked.
		EntityAddChild(entity_who_picked, perk_entity)
	end

	-- cosmetic fx -------------------------------------------------------
	if do_cosmetic_fx then
		local enemies_killed = tonumber( StatsBiomeGetValue("enemies_killed") )
		
		if( enemies_killed ~= 0 ) then
			EntityLoad( "data/entities/particles/image_emitters/perk_effect.xml", pos_x, pos_y )
		else
			EntityLoad( "data/entities/particles/image_emitters/perk_effect_pacifist.xml", pos_x, pos_y )
		end
		
		GamePrintImportant( GameTextGet( "$log_pickedup_perk", GameTextGetTranslatedOrNot( perk_name ) ), perk_desc )
	end
	
	-- remove all perk items (also this one!) ----------------------------
	if kill_other_perks then
		local perk_destroy_chance = tonumber( GlobalsGetValue( "TEMPLE_PERK_DESTROY_CHANCE", "100" ) )
		SetRandomSeed( pos_x, pos_y )

		if( Random( 1, 100 ) <= perk_destroy_chance ) then
			-- removes all the perks
			local all_perks = EntityGetWithTag( "perk" )
		
			if ( #all_perks > 0 ) then
				for i,entity_perk in ipairs(all_perks) do
					if entity_perk ~= entity_item then
						EntityKill( entity_perk )
					end
				end
			end
		end
	end

	EntityKill( entity_item ) -- entity item should always be killed, hence we don't kill it in the above loop
end